@@ -11,6 +11,7 @@ use eden_dag::ops::DagPersistent;
 use eden_dag::DagAlgorithm;
 use eyre::Context;
 use itertools::Itertools;
+use once_cell::sync::OnceCell;
 use tracing::{instrument, trace, warn};
 
 use crate::core::effects::{Effects, OperationType};
@@ -106,11 +107,16 @@ pub struct Dag {
 
     /// A set containing all commits that have been observed by the
     /// `EventReplayer`.
-    pub observed_commits: CommitSet,
+    observed_commits: CommitSet,
 
     /// A set containing all commits that have been determined to be obsolete by
     /// the `EventReplayer`.
-    pub obsolete_commits: CommitSet,
+    obsolete_commits: CommitSet,
+
+    public_commits: OnceCell<CommitSet>,
+    visible_heads: OnceCell<CommitSet>,
+    visible_commits: OnceCell<CommitSet>,
+    draft_commits: OnceCell<CommitSet>,
 }
 
 impl Dag {
@@ -167,10 +173,7 @@ impl Dag {
             })
             .collect();
 
-        let dag_dir = repo.get_dag_dir();
-        std::fs::create_dir_all(&dag_dir).wrap_err("Creating .git/branchless/dag dir")?;
-        let dag = eden_dag::Dag::open(&dag_dir)
-            .wrap_err_with(|| format!("Opening DAG directory at: {:?}", &dag_dir))?;
+        let dag = Self::open_inner_dag(repo)?;
 
         let observed_commits: CommitSet = observed_commits.into_iter().collect();
         let head_commit = match head_oid {
@@ -187,9 +190,22 @@ impl Dag {
             branch_commits,
             observed_commits,
             obsolete_commits,
+            public_commits: Default::default(),
+            visible_heads: Default::default(),
+            visible_commits: Default::default(),
+            draft_commits: Default::default(),
         })
     }
 
+    #[instrument]
+    fn open_inner_dag(repo: &Repo) -> eyre::Result<eden_dag::Dag> {
+        let dag_dir = repo.get_dag_dir();
+        std::fs::create_dir_all(&dag_dir).wrap_err("Creating .git/branchless/dag dir")?;
+        let dag = eden_dag::Dag::open(&dag_dir)
+            .wrap_err_with(|| format!("Opening DAG directory at: {:?}", &dag_dir))?;
+        Ok(dag)
+    }
+
     /// This function's code adapted from `GitDag`, licensed under GPL-2.
     #[instrument]
     fn sync(&mut self, effects: &Effects, repo: &Repo) -> eyre::Result<()> {
@@ -286,26 +302,22 @@ impl Dag {
         Ok(dag)
     }
 
-    /// Get one of the merge-base OIDs for the given pair of OIDs. If there are
-    /// multiple possible merge-bases, one is arbitrarily returned.
+    /// Create a new `Dag` with no obsolete commits.
     #[instrument]
-    pub fn get_one_merge_base_oid(
-        &self,
-        effects: &Effects,
-        repo: &Repo,
-        lhs_oid: NonZeroOid,
-        rhs_oid: NonZeroOid,
-    ) -> eyre::Result<Option<NonZeroOid>> {
-        let set = vec![CommitVertex::from(lhs_oid), CommitVertex::from(rhs_oid)];
-        let set = self
-            .inner
-            .sort(&CommitSet::from_static_names(set))
-            .wrap_err("Sorting DAG vertex set")?;
-        let vertex = self.inner.gca_one(set).wrap_err("Computing merge-base")?;
-        match vertex {
-            None => Ok(None),
-            Some(vertex) => Ok(Some(vertex.to_hex().parse()?)),
-        }
+    pub fn clear_obsolete_commits(&self, repo: &Repo) -> eyre::Result<Self> {
+        let inner = Self::open_inner_dag(repo)?;
+        Ok(Self {
+            inner,
+            head_commit: self.head_commit.clone(),
+            branch_commits: self.branch_commits.clone(),
+            main_branch_commit: self.main_branch_commit.clone(),
+            observed_commits: self.observed_commits.clone(),
+            obsolete_commits: CommitSet::empty(),
+            draft_commits: Default::default(),
+            public_commits: Default::default(),
+            visible_heads: Default::default(),
+            visible_commits: Default::default(),
+        })
     }
 
     /// Get the parent OID for the given OID. Returns an error if the given OID
@@ -361,65 +373,87 @@ impl Dag {
         self.inner.borrow()
     }
 
-    /// Return the set of commits which are public (checked into the main branch).
-    pub fn query_public_commits(&self) -> eyre::Result<CommitSet> {
-        let public_commits = self.query().ancestors(self.main_branch_commit.clone())?;
-        Ok(public_commits)
-    }
-
-    /// Query the set of active heads. This includes the heads of the set of
-    /// visible commits, plus any other commits which would be rendered in the
-    /// smartlog.
-    ///
-    /// This query includes heads which may be ancestor of other commits in the
-    /// set. For example, if `HEAD` points to a commit which is the ancestor of
-    /// a visible commit, both commits are included in the resulting set. This
-    /// is so that they can be explicitly rendered in the smartlog. To get the
-    /// set of visible commits, simply query for the ancestors of the set
-    /// resulting from this function.
-    pub fn query_active_heads(
-        &self,
-        public_commits: &CommitSet,
-        observed_commits: &CommitSet,
-    ) -> eyre::Result<CommitSet> {
-        let active_commits = observed_commits.clone();
-        let active_heads = self.query().heads(active_commits)?;
-        let active_heads = active_heads.difference(public_commits);
-
-        let anomalous_main_branch_commits = self.obsolete_commits.intersection(public_commits);
-        let active_heads = active_heads
-            .union(&self.head_commit)
-            .union(&self.branch_commits)
-            .union(&self.main_branch_commit)
-            .union(&anomalous_main_branch_commits);
-
-        Ok(active_heads)
-    }
-
-    /// Find a path from the provided head to its merge-base with the main
-    /// branch.
+    /// Determine whether or not the given commit is a public commit (i.e. is an
+    /// ancestor of the main branch).
     #[instrument]
-    pub fn find_path_to_main_branch(
-        &self,
-        effects: &Effects,
-        head: CommitSet,
-    ) -> eyre::Result<Option<CommitSet>> {
-        // FIXME: this assumes that there is only one merge-base with the main branch.
-        let merge_base = {
-            let (_effects, _progress) = effects.start_operation(OperationType::GetMergeBase);
-            self.query().gca_one(self.main_branch_commit.union(&head))?
-        };
-        let merge_base = match merge_base {
-            Some(merge_base) => merge_base,
-            None => return Ok(None),
-        };
+    pub fn is_public_commit(&self, commit_oid: NonZeroOid) -> eyre::Result<bool> {
+        let main_branch_commits = commit_set_to_vec(&self.main_branch_commit)?;
+        for main_branch_commit in main_branch_commits {
+            if self
+                .inner
+                .is_ancestor(commit_oid.into(), main_branch_commit.into())?
+            {
+                return Ok(true);
+            }
+        }
+        Ok(false)
+    }
 
-        // FIXME: this assumes that there is only one path to the merge-base.
-        let path = {
-            let (_effects, _progress) = effects.start_operation(OperationType::FindPathToMergeBase);
-            self.query().range(CommitSet::from(merge_base), head)?
-        };
-        Ok(Some(path))
+    /// Return the set of commits which are public, as per the definition in
+    /// `is_public_commit`. You should try to use `is_public_commit` instead, as
+    /// it will be faster to compute.
+    #[instrument]
+    pub fn query_public_commits_slow(&self) -> eyre::Result<&CommitSet> {
+        self.public_commits.get_or_try_init(|| {
+            let public_commits = self.query().ancestors(self.main_branch_commit.clone())?;
+            Ok(public_commits)
+        })
+    }
+
+    /// Determine the set of commits which are considered to be "visible". A
+    /// commit is "visible" if it is not obsolete or has a non-obsolete
+    /// descendant.
+    #[instrument]
+    pub fn query_visible_heads(&self) -> eyre::Result<&CommitSet> {
+        self.visible_heads.get_or_try_init(|| {
+            let visible_heads = CommitSet::empty()
+                .union(&self.observed_commits.difference(&self.obsolete_commits))
+                .union(&self.head_commit)
+                .union(&self.main_branch_commit)
+                .union(&self.branch_commits);
+            let visible_heads = self.query().heads(visible_heads)?;
+            Ok(visible_heads)
+        })
+    }
+
+    /// Query the set of all visible commits, as per the definition in
+    /// `query_visible_head`s. You should try to use `query_visible_heads`
+    /// instead if possible, since it will be faster to compute.
+    #[instrument]
+    pub fn query_visible_commits_slow(&self) -> eyre::Result<&CommitSet> {
+        self.visible_commits.get_or_try_init(|| {
+            let visible_heads = self.query_visible_heads()?;
+            let result = self.inner.ancestors(visible_heads.clone())?;
+            Ok(result)
+        })
+    }
+
+    /// Keep only commits in the given set which are visible, as per the
+    /// definition in `query_visible_heads`.
+    #[instrument]
+    pub fn filter_visible_commits(&self, commits: CommitSet) -> eyre::Result<CommitSet> {
+        let visible_heads = self.query_visible_heads()?;
+        Ok(commits.intersection(&self.query().range(commits.clone(), visible_heads.clone())?))
+    }
+
+    /// Determine the set of obsolete commits. These commits have been rewritten
+    /// or explicitly hidden by the user.
+    #[instrument]
+    pub fn query_obsolete_commits(&self) -> CommitSet {
+        self.obsolete_commits.clone()
+    }
+
+    /// Determine the set of "draft" commits. The draft commits are all visible
+    /// commits which aren't public.
+    #[instrument]
+    pub fn query_draft_commits(&self) -> eyre::Result<&CommitSet> {
+        self.draft_commits.get_or_try_init(|| {
+            let visible_heads = self.query_visible_heads()?;
+            let draft_commits = self
+                .query()
+                .only(visible_heads.clone(), self.main_branch_commit.clone())?;
+            Ok(draft_commits)
+        })
     }
 
     /// Given a CommitSet, return a list of CommitSets, each representing a
