@@ -19,7 +19,7 @@ use crate::core::task::ResourcePool;
 use crate::git::{Commit, NonZeroOid, PatchId, Repo};
 
 /// Represents the target for certain [`RebaseCommand`]s.
-#[derive(Debug)]
+#[derive(Clone, Debug)]
 pub enum OidOrLabel {
     /// A commit hash to check out directly.
     Oid(NonZeroOid),
@@ -68,10 +68,6 @@ pub enum RebaseCommand {
 
     /// Merge two or more parent commits.
     Merge {
-        /// If specified, the new merge commit will use this commit's message
-        /// and tree, rather than attempting a new merge.
-        replacement_commit_oid: Option<NonZeroOid>,
-
         /// The original merge commit to copy the commit contents from.
         commit_oid: NonZeroOid,
 
@@ -82,6 +78,22 @@ pub enum RebaseCommand {
         commits_to_merge: Vec<OidOrLabel>,
     },
 
+    /// When rebasing `commit_oid`, apply it by replacing its commit metadata
+    /// and tree with that of `replacement_commit_oid`, and by replacing its
+    /// parents with `parents`. No patches will be computed or applied, so this
+    /// operation is guaranteed to never cause a merge conflict.
+    Replace {
+        /// The commit to be replaced. It should be part of another rebase command, or else we will
+        /// never encounter it for replacement.
+        commit_oid: NonZeroOid,
+
+        /// The replacement commit whose metadata and tree we'll use.
+        replacement_commit_oid: NonZeroOid,
+
+        /// The new parents for the replaced commit.
+        parents: Vec<OidOrLabel>,
+    },
+
     /// Pause the rebase, to be resumed later. Only supported in on-disk
     /// rebases.
     Break,
@@ -121,14 +133,13 @@ pub struct RebasePlan {
 impl ToString for RebaseCommand {
     fn to_string(&self) -> String {
         match self {
-            RebaseCommand::CreateLabel { label_name } => format!("label {}", label_name),
-            RebaseCommand::Reset { target } => format!("reset {}", target),
+            RebaseCommand::CreateLabel { label_name } => format!("label {label_name}"),
+            RebaseCommand::Reset { target } => format!("reset {target}"),
             RebaseCommand::Pick {
                 original_commit_oid: _,
                 commit_to_apply_oid: commit_oid,
-            } => format!("pick {}", commit_oid),
+            } => format!("pick {commit_oid}"),
             RebaseCommand::Merge {
-                replacement_commit_oid: None,
                 commit_oid,
                 commits_to_merge,
             } => format!(
@@ -139,36 +150,35 @@ impl ToString for RebaseCommand {
                     .map(|commit_id| commit_id.to_string())
                     .join(" ")
             ),
-            RebaseCommand::Merge {
-                replacement_commit_oid: Some(replacement_commit_oid),
-                commit_oid,
-                commits_to_merge,
+            RebaseCommand::Replace {
+                commit_oid: _,
+                replacement_commit_oid,
+                parents,
             } => {
-                let other_parents = commits_to_merge
+                let parents = parents
                     .iter()
-                    .map(|parent| format!("-p {parent}"))
+                    .map(|parent| match parent {
+                        OidOrLabel::Oid(parent_oid) => format!("-p {parent_oid}"),
+                        OidOrLabel::Label(parent_label) => {
+                            format!("-p refs/rewritten/{parent_label}")
+                        }
+                    })
                     .join(" ");
                 // FIXME: Untested, this probably doesn't work. Currently,
                 // on-disk rebases for merges with replacement commits are not
                 // generated by any command, so this code path is never
                 // triggered.
-                format!("exec git show -s --format=%B {replacement_commit_oid} | git commit-tree -p {commit_oid} {other_parents} {replacement_commit_oid}^{{tree}}")
+                format!("exec git show -s --format=%B {replacement_commit_oid} | git commit-tree {parents} {replacement_commit_oid}^{{tree}}")
             }
             RebaseCommand::Break => "break".to_string(),
             RebaseCommand::RegisterExtraPostRewriteHook => {
                 "exec git branchless hook-register-extra-post-rewrite-hook".to_string()
             }
             RebaseCommand::DetectEmptyCommit { commit_oid } => {
-                format!(
-                    "exec git branchless hook-detect-empty-commit {}",
-                    commit_oid
-                )
+                format!("exec git branchless hook-detect-empty-commit {commit_oid}")
             }
             RebaseCommand::SkipUpstreamAppliedCommit { commit_oid } => {
-                format!(
-                    "exec git branchless hook-skip-upstream-applied-commit {}",
-                    commit_oid
-                )
+                format!("exec git branchless hook-skip-upstream-applied-commit {commit_oid}")
             }
         }
     }
@@ -176,16 +186,16 @@ impl ToString for RebaseCommand {
 
 /// A token representing that the rebase plan has been checked for validity.
 #[derive(Clone, Debug)]
-pub struct RebasePlanPermissions<'a> {
-    build_options: &'a BuildRebasePlanOptions,
+pub struct RebasePlanPermissions {
+    build_options: BuildRebasePlanOptions,
     allowed_commits: CommitSet,
 }
 
-impl<'a> RebasePlanPermissions<'a> {
+impl RebasePlanPermissions {
     /// Construct a new `RebasePlanPermissions`.
     pub fn verify_rewrite_set(
         dag: &Dag,
-        build_options: &'a BuildRebasePlanOptions,
+        build_options: BuildRebasePlanOptions,
         commits: &CommitSet,
     ) -> eyre::Result<Result<Self, BuildRebasePlanError>> {
         // This isn't necessary for correctness, but helps to produce a better
@@ -209,10 +219,7 @@ impl<'a> RebasePlanPermissions<'a> {
     }
 
     #[cfg(test)]
-    fn omnipotent_for_test(
-        dag: &Dag,
-        build_options: &'a BuildRebasePlanOptions,
-    ) -> eyre::Result<Self> {
+    fn omnipotent_for_test(dag: &Dag, build_options: BuildRebasePlanOptions) -> eyre::Result<Self> {
         Ok(Self {
             build_options,
             allowed_commits: dag.query().all()?,
@@ -224,7 +231,7 @@ impl<'a> RebasePlanPermissions<'a> {
 #[derive(Clone, Debug)]
 struct ConstraintGraph<'a> {
     dag: &'a Dag,
-    permissions: &'a RebasePlanPermissions<'a>,
+    permissions: &'a RebasePlanPermissions,
 
     /// This is a mapping from `x` to `y` if `x` must be applied before `y`
     inner: HashMap<NonZeroOid, HashSet<NonZeroOid>>,
@@ -505,7 +512,11 @@ struct BuildState<'repo> {
     /// we first have to make sure that all of its parents have been committed.
     /// (If not, then we stop this sub-traversal and wait for a later traversal
     /// to hit the same merge commit).
-    merge_commit_parent_labels: HashMap<NonZeroOid, String>,
+    ///
+    /// When replacing commits and specifying a parent which is also being
+    /// rebased, we need to refer to it via label instead of OID, to ensure that
+    /// we use the rewritten parent.
+    parent_labels: HashMap<NonZeroOid, String>,
 }
 
 /// Builder for a rebase plan. Unlike regular Git rebases, a `git-branchless`
@@ -513,7 +524,7 @@ struct BuildState<'repo> {
 #[derive(Clone, Debug)]
 pub struct RebasePlanBuilder<'a> {
     dag: &'a Dag,
-    permissions: RebasePlanPermissions<'a>,
+    permissions: RebasePlanPermissions,
 
     /// The constraints specified by the caller.
     initial_constraints: Vec<Constraint>,
@@ -671,7 +682,7 @@ This is a bug. Please report it.",
 
 impl<'a> RebasePlanBuilder<'a> {
     /// Constructor.
-    pub fn new(dag: &'a Dag, permissions: RebasePlanPermissions<'a>) -> Self {
+    pub fn new(dag: &'a Dag, permissions: RebasePlanPermissions) -> Self {
         RebasePlanBuilder {
             dag,
             permissions,
@@ -701,7 +712,6 @@ impl<'a> RebasePlanBuilder<'a> {
         effects: &Effects,
         repo: &Repo,
         state: &mut BuildState,
-        previous_head_oid: NonZeroOid,
         current_commit: Commit,
         upstream_patch_ids: &HashSet<PatchId>,
         mut acc: Vec<RebaseCommand>,
@@ -734,11 +744,6 @@ impl<'a> RebasePlanBuilder<'a> {
                 let commits_to_merge: Option<Vec<OidOrLabel>> = current_commit
                     .get_parent_oids()
                     .into_iter()
-                    .filter(|parent_oid| {
-                        // Don't include the parent which was the previous HEAD
-                        // commit, since we need to merge into that commit.
-                        *parent_oid != previous_head_oid
-                    })
                     .map(|parent_oid| -> Option<OidOrLabel> {
                         let does_parent_commit_need_rebase =
                             state.constraints.commits_to_move().contains(&parent_oid);
@@ -752,9 +757,9 @@ impl<'a> RebasePlanBuilder<'a> {
                             // that not all parents have been applied yet, so
                             // we'll postpone this commit.
                             state
-                                .merge_commit_parent_labels
+                                .parent_labels
                                 .get(&parent_oid)
-                                .map(|parent_oid| OidOrLabel::Label(parent_oid.clone()))
+                                .map(|parent_label| OidOrLabel::Label(parent_label.clone()))
                         } else {
                             // This parent commit was not supposed to be
                             // rebased, so its OID won't change and we can
@@ -764,61 +769,102 @@ impl<'a> RebasePlanBuilder<'a> {
                     })
                     .collect();
 
-                match commits_to_merge {
+                let commits_to_merge = match commits_to_merge {
                     Some(commits_to_merge) => {
-                        if acc.iter().any(|command| {
-                            matches!(command,
+                        // We've already checked to make sure that all parent
+                        // commits which are being moved have had their labels
+                        // applied. However, if the commit has multiple parents
+                        // which were *not* moved, then we would visit this
+                        // commit and attempt to reapply it multiple times. The
+                        // simplest fix is to not reapply merge commits which
+                        // have already been applied.
+                        //
+                        // FIXME: O(n^2) algorithm.
+                        let commit_already_applied = acc.iter().any(|command| match command {
                             RebaseCommand::Merge {
-                                replacement_commit_oid: _,
                                 commit_oid,
                                 commits_to_merge: _,
-                            } if *commit_oid == current_commit.get_oid())
-                        }) {
-                            // Above, we've checked to make sure that all parent
-                            // commits which are being moved have had their
-                            // labels applied. However, if the commit has
-                            // multiple parents which were *not* moved, then we
-                            // would visit this commit and attempt to reapply it
-                            // multiple times. The simplest fix is to not
-                            // reapply merge commits which have already been
-                            // applied.
-                            //
-                            // FIXME: O(n^2) algorithm.
-                            return Ok(acc);
+                            }
+                            | RebaseCommand::Replace {
+                                commit_oid,
+                                replacement_commit_oid: _,
+                                parents: _,
+                            } => *commit_oid == current_commit.get_oid(),
+                            _ => false,
+                        });
+                        if !commit_already_applied {
+                            Some(commits_to_merge)
                         } else {
-                            // All parents have been committed.
-                            acc.push(RebaseCommand::Merge {
-                                replacement_commit_oid: self
-                                    .replacement_commits
-                                    .get(&current_commit.get_oid())
-                                    .copied(),
-                                commit_oid: current_commit.get_oid(),
-                                commits_to_merge,
-                            });
+                            None
                         }
                     }
 
                     None => {
-                        // Wait for the caller to come back to this commit
-                        // later and then proceed to any child commits.
-                        return Ok(acc);
+                        // Wait for the caller to come back to this commit later
+                        // and then proceed to any child commits.
+                        None
                     }
+                };
+
+                if let Some(commits_to_merge) = commits_to_merge {
+                    // All parents have been committed.
+                    let (first_parent, merge_parents) = match commits_to_merge.as_slice() {
+                        [] => {
+                            unreachable!("Already verified that there's at least one parent commit")
+                        }
+                        [first, rest @ ..] => (first, rest.to_vec()),
+                    };
+                    acc.push(RebaseCommand::Reset {
+                        target: first_parent.clone(),
+                    });
+                    acc.push(
+                        match self.replacement_commits.get(&current_commit.get_oid()) {
+                            None => RebaseCommand::Merge {
+                                commit_oid: current_commit.get_oid(),
+                                commits_to_merge: merge_parents.to_vec(),
+                            },
+                            Some(replacement_commit_oid) => RebaseCommand::Replace {
+                                commit_oid: current_commit.get_oid(),
+                                replacement_commit_oid: *replacement_commit_oid,
+                                parents: commits_to_merge,
+                            },
+                        },
+                    );
                 }
+
+                return Ok(acc);
             } else {
                 // Normal one-parent commit (or a zero-parent commit?), just
                 // rebase it and continue.
                 let original_commit_oid = current_commit.get_oid();
-                let commit_oid = match self.replacement_commits.get(&original_commit_oid) {
-                    Some(replacement_oid) => *replacement_oid,
-                    None => original_commit_oid,
+                match self.replacement_commits.get(&original_commit_oid) {
+                    Some(replacement_commit_oid) => {
+                        let replacement_commit =
+                            repo.find_commit_or_fail(*replacement_commit_oid)?;
+                        let new_parents = replacement_commit
+                            .get_parent_oids()
+                            .into_iter()
+                            .map(|parent_oid| match state.parent_labels.get(&parent_oid) {
+                                None => OidOrLabel::Oid(parent_oid),
+                                Some(parent_label) => OidOrLabel::Label(parent_label.clone()),
+                            })
+                            .collect_vec();
+                        acc.push(RebaseCommand::Replace {
+                            commit_oid: original_commit_oid,
+                            replacement_commit_oid: *replacement_commit_oid,
+                            parents: new_parents,
+                        });
+                    }
+                    None => {
+                        acc.push(RebaseCommand::Pick {
+                            original_commit_oid,
+                            commit_to_apply_oid: original_commit_oid,
+                        });
+                        acc.push(RebaseCommand::DetectEmptyCommit {
+                            commit_oid: current_commit.get_oid(),
+                        });
+                    }
                 };
-                acc.push(RebaseCommand::Pick {
-                    original_commit_oid,
-                    commit_to_apply_oid: commit_oid,
-                });
-                acc.push(RebaseCommand::DetectEmptyCommit {
-                    commit_oid: current_commit.get_oid(),
-                });
             }
             acc
         };
@@ -835,18 +881,19 @@ impl<'a> RebasePlanBuilder<'a> {
             })?;
 
         let acc = {
-            if child_commits
-                .iter()
-                .any(|child_commit| child_commit.get_parent_count() > 1)
-            {
-                // If this commit has any merge commits as children, create a
-                // label so that the child can reference this commit later for
-                // merging.
+            if child_commits.iter().any(|child_commit| {
+                child_commit.get_parent_count() > 1
+                    || self
+                        .replacement_commits
+                        .contains_key(&child_commit.get_oid())
+            }) {
+                // If this commit has any merge commits or replaced commits as
+                // children, create a label so that the child can reference this
+                // commit later for merging.
                 let command_num = acc.len();
-                let label_name =
-                    self.make_label_name(state, format!("merge-parent-{}", command_num));
+                let label_name = self.make_label_name(state, format!("parent-{command_num}"));
                 state
-                    .merge_commit_parent_labels
+                    .parent_labels
                     .insert(current_commit.get_oid(), label_name.clone());
                 let mut acc = acc;
                 acc.push(RebaseCommand::CreateLabel { label_name });
@@ -865,7 +912,6 @@ impl<'a> RebasePlanBuilder<'a> {
                 effects,
                 repo,
                 state,
-                current_commit.get_oid(),
                 only_child_commit,
                 upstream_patch_ids,
                 acc,
@@ -873,7 +919,7 @@ impl<'a> RebasePlanBuilder<'a> {
             Ok(acc)
         } else {
             let command_num = acc.len();
-            let label_name = self.make_label_name(state, format!("label-{}", command_num));
+            let label_name = self.make_label_name(state, format!("label-{command_num}"));
             let mut acc = acc;
             acc.push(RebaseCommand::CreateLabel {
                 label_name: label_name.clone(),
@@ -883,7 +929,6 @@ impl<'a> RebasePlanBuilder<'a> {
                     effects,
                     repo,
                     state,
-                    current_commit.get_oid(),
                     child_commit,
                     upstream_patch_ids,
                     acc,
@@ -973,7 +1018,7 @@ impl<'a> RebasePlanBuilder<'a> {
         let mut state = BuildState {
             constraints,
             used_labels: Default::default(),
-            merge_commit_parent_labels: Default::default(),
+            parent_labels: Default::default(),
         };
 
         let (effects, _progress) = effects.start_operation(OperationType::BuildRebasePlan);
@@ -983,7 +1028,7 @@ impl<'a> RebasePlanBuilder<'a> {
             dump_rebase_constraints,
             dump_rebase_plan,
             detect_duplicate_commits_via_patch_id,
-        } = self.permissions.build_options;
+        } = &self.permissions.build_options;
         if *dump_rebase_constraints {
             // For test: don't print to `effects.get_output_stream()`, as it will
             // be suppressed.
@@ -1049,7 +1094,6 @@ impl<'a> RebasePlanBuilder<'a> {
                 &effects,
                 &repo,
                 &mut state,
-                first_parent_oid,
                 repo.find_commit_or_fail(child_oid)?,
                 &upstream_patch_ids,
                 acc,
@@ -1067,7 +1111,7 @@ impl<'a> RebasePlanBuilder<'a> {
         if *dump_rebase_plan {
             // For test: don't print to `effects.get_output_stream()`, as it will
             // be suppressed.
-            println!("Rebase plan: {:#?}", rebase_plan);
+            println!("Rebase plan: {rebase_plan:#?}");
         }
         Ok(Ok(rebase_plan))
     }
@@ -1089,10 +1133,14 @@ impl<'a> RebasePlanBuilder<'a> {
                     commit_to_apply_oid,
                 } => vec![*original_commit_oid, *commit_to_apply_oid],
                 RebaseCommand::Merge {
-                    replacement_commit_oid: _,
                     commit_oid,
                     commits_to_merge: _,
                 }
+                | RebaseCommand::Replace {
+                    commit_oid,
+                    replacement_commit_oid: _,
+                    parents: _,
+                }
                 | RebaseCommand::SkipUpstreamAppliedCommit { commit_oid } => vec![*commit_oid],
             })
             .collect();
@@ -1350,7 +1398,7 @@ mod tests {
             dump_rebase_plan: false,
             detect_duplicate_commits_via_patch_id: true,
         };
-        let permissions = RebasePlanPermissions::omnipotent_for_test(&dag, &build_options)?;
+        let permissions = RebasePlanPermissions::omnipotent_for_test(&dag, build_options)?;
         let pool = ThreadPoolBuilder::new().build()?;
         let repo_pool = RepoPool::new(RepoResource {
             repo: Mutex::new(repo.try_clone()?),
@@ -1383,7 +1431,7 @@ mod tests {
             Ok(())
         })?;
 
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
         insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
@@ -1415,7 +1463,7 @@ mod tests {
             Ok(())
         })?;
 
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
         insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
@@ -1447,7 +1495,7 @@ mod tests {
             Ok(())
         })?;
 
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
         insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
@@ -1479,7 +1527,7 @@ mod tests {
             Ok(())
         })?;
 
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
         insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
@@ -1515,7 +1563,7 @@ mod tests {
             Ok(())
         })?;
 
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
         insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
@@ -1552,7 +1600,7 @@ mod tests {
             Ok(())
         })?;
 
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
         insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
@@ -1587,7 +1635,7 @@ mod tests {
             Ok(())
         })?;
 
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
         insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
@@ -1618,7 +1666,7 @@ mod tests {
         git.run(&["checkout", &test3_oid.to_string()])?;
         git.commit_file("test5", 5)?;
 
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
         insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
@@ -1639,7 +1687,7 @@ mod tests {
             Ok(())
         })?;
 
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
         insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
@@ -1674,7 +1722,7 @@ mod tests {
             Ok(())
         })?;
 
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
         insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
@@ -1710,7 +1758,7 @@ mod tests {
             Ok(())
         })?;
 
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
         insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
@@ -1748,7 +1796,7 @@ mod tests {
             Ok(())
         })?;
 
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
 
         // FIXME This output is correct mathematically, but feels like it should
         // be incorrect. What *should* we be doing if the user moves 2 ranges w/
@@ -1793,7 +1841,7 @@ mod tests {
             Ok(())
         })?;
 
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
         insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
@@ -1830,7 +1878,7 @@ mod tests {
             Ok(())
         })?;
 
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
         insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
@@ -1865,7 +1913,7 @@ mod tests {
             Ok(())
         })?;
 
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
         insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
@@ -1900,7 +1948,7 @@ mod tests {
             Ok(())
         })?;
 
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
         insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
@@ -1938,7 +1986,7 @@ mod tests {
         })?;
 
         // FIXME: this is wrong, the merge commit should be moved as well.
-        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        let stdout = git.smartlog()?;
         insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
@@ -2002,7 +2050,7 @@ mod tests {
             dump_rebase_plan: false,
             detect_duplicate_commits_via_patch_id: true,
         };
-        let permissions = RebasePlanPermissions::omnipotent_for_test(&dag, &build_options)?;
+        let permissions = RebasePlanPermissions::omnipotent_for_test(&dag, build_options)?;
         let mut builder = RebasePlanBuilder::new(&dag, permissions);
 
         builder_callback_fn(&mut builder)?;
@@ -2027,6 +2075,7 @@ mod tests {
             resolve_merge_conflicts: false,
             check_out_commit_options: CheckOutCommitOptions {
                 additional_args: Default::default(),
+                reset: false,
                 render_smartlog: false,
             },
         };
