@@ -100,7 +100,7 @@ pub enum Error {
     CreateBlob(#[source] git2::Error),
 
     #[error("could not create blob from {path}: {source}")]
-    CreateBlobFromPath { source: git2::Error, path: PathBuf },
+    CreateBlobFromPath { source: eyre::Error, path: PathBuf },
 
     #[error("could not find commit {oid}: {source}")]
     FindCommit {
@@ -167,6 +167,9 @@ pub enum Error {
         name: ReferenceName,
     },
 
+    #[error("could not delete branch: {0}")]
+    DeleteBranch(#[source] git2::Error),
+
     #[error("could not delete reference: {0}")]
     DeleteReference(#[source] git2::Error),
 
@@ -578,12 +581,6 @@ impl Repo {
         self.get_path().join("branchless").join("man")
     }
 
-    /// Get the directory where the results of running tests are stored.
-    #[instrument]
-    pub fn get_test_dir(&self) -> PathBuf {
-        self.get_path().join("branchless").join("test")
-    }
-
     /// Get a directory suitable for storing temporary files.
     ///
     /// In particular, this directory is guaranteed to be on the same filesystem
@@ -816,7 +813,6 @@ impl Repo {
             })?;
         let paths = diff
             .deltas()
-            .into_iter()
             .flat_map(|delta| vec![delta.old_file().path(), delta.new_file().path()])
             .flat_map(|p| p.map(PathBuf::from))
             .collect();
@@ -1110,7 +1106,7 @@ impl Repo {
             None => {
                 let NonZeroOid { inner: oid } = oid;
                 Ok(StyledString::styled(
-                    format!("<commit not available: {}>", oid),
+                    format!("<commit not available: {oid}>"),
                     BaseColor::Red.light(),
                 ))
             }
@@ -1121,14 +1117,30 @@ impl Repo {
     /// If the file doesn't exist on disk, returns `None` instead.
     #[instrument]
     pub fn create_blob_from_path(&self, path: &Path) -> Result<Option<NonZeroOid>> {
-        match self.inner.blob_path(path) {
-            Ok(oid) => Ok(Some(make_non_zero_oid(oid))),
-            Err(err) if err.code() == git2::ErrorCode::NotFound => Ok(None),
-            Err(err) => Err(Error::CreateBlobFromPath {
-                source: err,
-                path: path.to_owned(),
-            }),
-        }
+        // Can't use `self.inner.blob_path`, because it will read the file from
+        // the main repository instead of from the current worktree.
+        let path = self
+            .get_working_copy_path()
+            .ok_or_else(|| Error::CreateBlobFromPath {
+                source: eyre::eyre!(
+                    "Repository at {:?} has no working copy path (is bare)",
+                    self.get_path()
+                ),
+                path: path.to_path_buf(),
+            })?
+            .join(path);
+        let contents = match std::fs::read(&path) {
+            Ok(contents) => contents,
+            Err(err) if err.kind() == std::io::ErrorKind::NotFound => return Ok(None),
+            Err(err) => {
+                return Err(Error::CreateBlobFromPath {
+                    source: err.into(),
+                    path,
+                })
+            }
+        };
+        let blob = self.create_blob_from_contents(&contents)?;
+        Ok(Some(blob))
     }
 
     /// Create a blob corresponding to the provided byte slice.
@@ -1208,7 +1220,7 @@ impl Repo {
 
         if *reuse_parent_tree_if_possible {
             if let Some(only_parent) = patch_commit.get_only_parent() {
-                if only_parent.get_tree()?.get_oid() == target_commit.get_tree()?.get_oid() {
+                if only_parent.get_tree_oid() == target_commit.get_tree_oid() {
                     // If this patch is being applied to the same commit it was
                     // originally based on, then we can skip cherry-picking
                     // altogether, and use its tree directly. This is common e.g.
@@ -1577,9 +1589,9 @@ impl<'repo> Signature<'repo> {
     /// Return the friendly formatted name and email of the signature.
     pub fn friendly_describe(&self) -> Option<String> {
         let name = self.inner.name();
-        let email = self.inner.email().map(|email| format!("<{}>", email));
+        let email = self.inner.email().map(|email| format!("<{email}>"));
         match (name, email) {
-            (Some(name), Some(email)) => Some(format!("{} {}", name, email)),
+            (Some(name), Some(email)) => Some(format!("{name} {email}")),
             (Some(name), _) => Some(name.into()),
             (_, Some(email)) => Some(email),
             _ => None,
@@ -1612,8 +1624,8 @@ impl Time {
     }
 
     /// Calculate the associated [`NaiveDateTime`].
-    pub fn to_naive_date_time(&self) -> NaiveDateTime {
-        NaiveDateTime::from_timestamp(self.inner.seconds(), 0)
+    pub fn to_naive_date_time(&self) -> Option<NaiveDateTime> {
+        NaiveDateTime::from_timestamp_opt(self.inner.seconds(), 0)
     }
 }
 
