@@ -13,7 +13,7 @@ use eyre::{eyre, Context};
 use itertools::Itertools;
 use tracing::instrument;
 
-use crate::core::config::get_core_hooks_path;
+use crate::core::config::get_hooks_dir;
 use crate::core::effects::{Effects, OperationType};
 use crate::core::eventlog::{EventTransactionId, BRANCHLESS_TRANSACTION_ID_ENV_VAR};
 use crate::git::repo::Repo;
@@ -103,7 +103,7 @@ impl GitRunInfo {
             let reader = BufReader::new(stream);
             for line in reader.lines() {
                 let line = line.expect("Reading line from subprocess");
-                writeln!(output, "{}", line).expect("Writing line from subprocess");
+                writeln!(output, "{line}").expect("Writing line from subprocess");
             }
         })
     }
@@ -125,7 +125,7 @@ impl GitRunInfo {
             .map(|arg| arg.to_string_lossy().to_string())
             .collect_vec()
             .join(" ");
-        let command_string = format!("git {}", args_string);
+        let command_string = format!("git {args_string}");
         let (effects, _progress) =
             effects.start_operation(OperationType::RunGitCommand(Arc::new(command_string)));
         writeln!(
@@ -351,7 +351,7 @@ impl GitRunInfo {
         args: &[&str],
         stdin: Option<BString>,
     ) -> eyre::Result<()> {
-        let hook_dir = get_core_hooks_path(repo)?;
+        let hook_dir = get_hooks_dir(self, repo, Some(event_tx_id))?;
         if !hook_dir.exists() {
             return Ok(());
         }
@@ -383,7 +383,7 @@ impl GitRunInfo {
                         .unwrap_or_else(|| repo.get_path()),
                 )
                 .arg("-c")
-                .arg(format!("{} \"$@\"", hook_name))
+                .arg(format!("{hook_name} \"$@\""))
                 .arg(hook_name) // "$@" expands "$1" "$2" "$3" ... but we also must specify $0.
                 .args(args)
                 .env_clear()
