@@ -11,16 +11,21 @@ use std::ops::Deref;
 use std::path::{Path, PathBuf};
 use std::process::{Command, Stdio};
 
-use crate::core::config::env_vars::{get_git_exec_path, get_path_to_git, TEST_GIT};
+use crate::core::config::env_vars::{
+    get_git_exec_path, get_path_to_git, should_use_separate_command_binary, TEST_GIT,
+    TEST_SEPARATE_COMMAND_BINARIES,
+};
 use crate::git::{GitRunInfo, GitVersion, NonZeroOid, Repo};
 use crate::util::get_sh;
 
+use color_eyre::Help;
 use eyre::Context;
 use itertools::Itertools;
 use lazy_static::lazy_static;
+use once_cell::sync::OnceCell;
 use regex::{Captures, Regex};
 use tempfile::TempDir;
-use tracing::instrument;
+use tracing::{instrument, warn};
 
 const DUMMY_NAME: &str = "Testy McTestface";
 const DUMMY_EMAIL: &str = "test@example.com";
@@ -154,7 +159,7 @@ impl Git {
     pub fn get_base_env(&self, time: isize) -> Vec<(OsString, OsString)> {
         // Required for determinism, as these values will be baked into the commit
         // hash.
-        let date: OsString = format!("{date} -{time:0>2}", date = DUMMY_DATE, time = time).into();
+        let date: OsString = format!("{DUMMY_DATE} -{time:0>2}").into();
 
         // Fake "editor" which accepts the default contents of any commit
         // messages. Usually, we can set this with `git commit -m`, but we have
@@ -173,6 +178,10 @@ impl Git {
             ("GIT_EXEC_PATH", self.git_exec_path.as_os_str().into()),
             ("PATH", new_path),
             (TEST_GIT, self.path_to_git.as_os_str().into()),
+            (
+                TEST_SEPARATE_COMMAND_BINARIES,
+                std::env::var_os(TEST_SEPARATE_COMMAND_BINARIES).unwrap_or_default(),
+            ),
         ];
 
         envs.into_iter()
@@ -285,9 +294,74 @@ stderr:
         &self,
         args: &[S],
     ) -> eyre::Result<(String, String)> {
+        if let Some(first_arg) = args.first() {
+            if first_arg.as_ref() == "branchless" {
+                eyre::bail!(
+                    r#"Refusing to invoke `branchless` via `git.run(&["branchless", ...])`; instead, call `git.branchless(&[...])`"#
+                );
+            }
+        }
+
         self.run_with_options(args, &Default::default())
     }
 
+    /// Render the smartlog for the repository.
+    #[instrument]
+    pub fn smartlog(&self) -> eyre::Result<String> {
+        let (stdout, _stderr) = self.branchless("smartlog", &[])?;
+        Ok(stdout)
+    }
+
+    /// Convenience method to call `branchless_with_options` with the default
+    /// options.
+    #[instrument]
+    pub fn branchless(&self, subcommand: &str, args: &[&str]) -> eyre::Result<(String, String)> {
+        self.branchless_with_options(subcommand, args, &Default::default())
+    }
+
+    /// Locate the git-branchless binary and run a git-branchless subcommand
+    /// with the provided `GitRunOptions`. These subcommands are located using
+    /// `should_use_separate_command_binary`.
+    #[instrument]
+    pub fn branchless_with_options(
+        &self,
+        subcommand: &str,
+        args: &[&str],
+        options: &GitRunOptions,
+    ) -> eyre::Result<(String, String)> {
+        let mut git_run_args = Vec::new();
+        if should_use_separate_command_binary(subcommand) {
+            git_run_args.push(format!("branchless-{subcommand}"));
+        } else {
+            git_run_args.push("branchless".to_string());
+            git_run_args.push(subcommand.to_string());
+        }
+        git_run_args.extend(args.iter().map(|arg| arg.to_string()));
+
+        let result = self.run_with_options(&git_run_args, options);
+
+        if !should_use_separate_command_binary(subcommand) {
+            result
+        } else {
+            result.suggestion(format!(
+                "\
+If you have set the TEST_SEPARATE_COMMAND_BINARIES environment variable, then \
+the git-branchless-{subcommand} binary is NOT automatically built or updated when \
+running integration tests for other binaries (see \
+https://github.com/rust-lang/cargo/issues/4316 for more details).
+
+Make sure that git-branchless-{subcommand} has been built before running \
+integration tests. You can build it with: cargo build -p
+git-branchless-{subcommand}
+
+If you have not set the TEST_SEPARATE_COMMAND_BINARIES environment variable, \
+then you can only run tests in the main `git-branchless` and \
+`git-branchless-lib` crates.\
+        ",
+            ))
+        }
+    }
+
     /// Set up a Git repo in the directory and initialize git-branchless to work
     /// with it.
     #[instrument]
@@ -317,7 +391,7 @@ stderr:
         self.run(&["config", "core.autocrlf", "false"])?;
 
         if options.run_branchless_init {
-            self.run(&["branchless", "init"])?;
+            self.branchless("init", &[])?;
         }
 
         Ok(())
@@ -370,7 +444,7 @@ stderr:
 
     /// Delete the provided file in the repository root.
     pub fn delete_file(&self, name: &str) -> eyre::Result<()> {
-        let file_path = self.repo_path.join(format!("{}.txt", name));
+        let file_path = self.repo_path.join(format!("{name}.txt"));
         fs::remove_file(file_path)?;
         Ok(())
     }
@@ -381,7 +455,7 @@ stderr:
         name: &str,
         permissions: fs::Permissions,
     ) -> eyre::Result<()> {
-        let file_path = self.repo_path.join(format!("{}.txt", name));
+        let file_path = self.repo_path.join(format!("{name}.txt"));
         fs::set_permissions(file_path, permissions)?;
         Ok(())
     }
@@ -398,7 +472,7 @@ stderr:
         self.write_file_txt(name, contents)?;
         self.run(&["add", "."])?;
         self.run_with_options(
-            &["commit", "-m", &format!("create {}.txt", name)],
+            &["commit", "-m", &format!("create {name}.txt")],
             &GitRunOptions {
                 time,
                 ..Default::default()
@@ -416,7 +490,7 @@ stderr:
     /// Commit a file with default contents. The `time` argument is used to set
     /// the commit timestamp, which is factored into the commit hash.
     pub fn commit_file(&self, name: &str, time: isize) -> eyre::Result<NonZeroOid> {
-        self.commit_file_with_contents(name, time, &format!("{} contents\n", name))
+        self.commit_file_with_contents(name, time, &format!("{name} contents\n"))
     }
 
     /// Detach HEAD. This is useful to call to make sure that no branch is
@@ -498,7 +572,7 @@ stderr:
     /// contents.
     #[instrument]
     pub fn resolve_file(&self, name: &str, contents: &str) -> eyre::Result<()> {
-        let file_path = self.repo_path.join(format!("{}.txt", name));
+        let file_path = self.repo_path.join(format!("{name}.txt"));
         std::fs::write(&file_path, contents)?;
         let file_path = match file_path.to_str() {
             None => eyre::bail!("Could not convert file path to string: {:?}", file_path),
@@ -566,8 +640,12 @@ impl GitWrapper {
     }
 }
 
+static COLOR_EYRE_INSTALL: OnceCell<()> = OnceCell::new();
+
 /// Create a temporary directory for testing and a `Git` instance to use with it.
 pub fn make_git() -> eyre::Result<GitWrapper> {
+    COLOR_EYRE_INSTALL.get_or_try_init(color_eyre::install)?;
+
     let repo_dir = tempfile::tempdir()?;
     let path_to_git = get_path_to_git()?;
     let git_exec_path = get_git_exec_path()?;
@@ -633,3 +711,199 @@ pub fn make_git_worktree(git: &Git, worktree_name: &str) -> eyre::Result<GitWork
     };
     Ok(GitWorktreeWrapper { temp_dir, worktree })
 }
+
+/// Find and extract the command to disable the hint mentioned in the output.
+/// Returns the arguments to `git` which would disable the hint.
+pub fn extract_hint_command(stdout: &str) -> Vec<String> {
+    let hint_command = stdout
+        .split_once("disable this hint by running: ")
+        .map(|(_first, second)| second)
+        .unwrap()
+        .split('\n')
+        .next()
+        .unwrap();
+    hint_command
+        .split(' ')
+        .skip(1) // "git"
+        .filter(|s| s != &"--global")
+        .map(|s| s.to_owned())
+        .collect_vec()
+}
+
+/// Remove some of the output from `git rebase`, as it seems to be
+/// non-deterministic as to whether or not it appears.
+pub fn remove_rebase_lines(output: String) -> String {
+    output
+        .lines()
+        .filter(|line| !line.contains("First, rewinding head") && !line.contains("Applying:"))
+        .filter(|line| {
+            // See https://github.com/arxanas/git-branchless/issues/87.  Before
+            // Git v2.33 (`next` branch), the "Auto-merging" line appears
+            // *after* the "CONFLICT" line for a given file (which doesn't make
+            // sense -- how can there be a conflict before merging has started)?
+            // The development version of Git v2.33 fixes this and places the
+            // "Auto-merging" line *before* the "CONFLICT" line. To avoid having
+            // to deal with multiple possible output formats, just remove the
+            // line in question.
+            !line.contains("Auto-merging")
+        })
+        .map(|line| format!("{line}\n"))
+        .collect()
+}
+
+/// Remove whitespace from the end of each line in the provided string.
+pub fn trim_lines(output: String) -> String {
+    output
+        .lines()
+        .flat_map(|line| vec![line.trim_end(), "\n"].into_iter())
+        .collect()
+}
+
+/// Utilities for testing in a virtual terminal (PTY).
+pub mod pty {
+    use std::{
+        sync::{mpsc::channel, Arc, Mutex},
+        thread,
+        time::Duration,
+    };
+
+    use eyre::eyre;
+    use portable_pty::{native_pty_system, CommandBuilder, PtySize};
+
+    use super::Git;
+
+    /// An action to take as part of the PTY test script.
+    pub enum PtyAction<'a> {
+        /// Input the provided string as keystrokes to the terminal.
+        Write(&'a str),
+
+        /// Wait until the terminal display shows the provided string anywhere
+        /// on the screen.
+        WaitUntilContains(&'a str),
+    }
+
+    /// Run the provided script in the context of a virtual terminal.
+    pub fn run_in_pty(
+        git: &Git,
+        branchless_subcommand: &str,
+        args: &[&str],
+        inputs: &[PtyAction],
+    ) -> eyre::Result<()> {
+        // Use the native pty implementation for the system
+        let pty_system = native_pty_system();
+        let pty_size = PtySize::default();
+        let mut pty = pty_system
+            .openpty(pty_size)
+            .map_err(|e| eyre!("Could not open pty: {}", e))?;
+
+        // Spawn a git instance in the pty.
+        let mut cmd = CommandBuilder::new(&git.path_to_git);
+        cmd.env_clear();
+        for (k, v) in git.get_base_env(0) {
+            cmd.env(k, v);
+        }
+        cmd.env("TERM", "xterm");
+        cmd.arg("branchless");
+        cmd.arg(branchless_subcommand);
+        cmd.args(args);
+        cmd.cwd(&git.repo_path);
+
+        let mut child = pty
+            .slave
+            .spawn_command(cmd)
+            .map_err(|e| eyre!("Could not spawn child: {}", e))?;
+
+        let reader = pty
+            .master
+            .try_clone_reader()
+            .map_err(|e| eyre!("Could not clone reader: {}", e))?;
+        let reader = Arc::new(Mutex::new(reader));
+
+        let parser = vt100::Parser::new(pty_size.rows, pty_size.cols, 0);
+        let parser = Arc::new(Mutex::new(parser));
+
+        for action in inputs {
+            match action {
+                PtyAction::WaitUntilContains(value) => {
+                    let (finished_tx, finished_rx) = channel();
+
+                    let wait_thread = {
+                        let parser = Arc::clone(&parser);
+                        let reader = Arc::clone(&reader);
+                        let value = value.to_string();
+                        thread::spawn(move || -> anyhow::Result<()> {
+                            loop {
+                                // Drop the `parser` lock after this, since we may block
+                                // on `reader.read` below, and the caller may want to
+                                // check the screen contents of `parser`.
+                                {
+                                    let parser = parser.lock().unwrap();
+                                    if parser.screen().contents().contains(&value) {
+                                        break;
+                                    }
+                                }
+
+                                let mut reader = reader.lock().unwrap();
+                                const BUF_SIZE: usize = 4096;
+                                let mut buffer = [0; BUF_SIZE];
+                                let n = reader.read(&mut buffer)?;
+                                assert!(n < BUF_SIZE, "filled up PTY buffer by reading {n} bytes",);
+
+                                {
+                                    let mut parser = parser.lock().unwrap();
+                                    parser.process(&buffer[..n]);
+                                }
+                            }
+
+                            finished_tx.send(()).unwrap();
+                            Ok(())
+                        })
+                    };
+
+                    if finished_rx.recv_timeout(Duration::from_secs(5)).is_err() {
+                        panic!(
+                            "\
+Timed out waiting for virtual terminal to show string: {:?}
+Screen contents:
+-----
+{}
+-----
+",
+                            value,
+                            parser.lock().unwrap().screen().contents(),
+                        );
+                    }
+
+                    wait_thread.join().unwrap().unwrap();
+                }
+
+                PtyAction::Write(value) => {
+                    write!(pty.master, "{value}")?;
+                    pty.master.flush()?;
+                }
+            }
+        }
+
+        let read_remainder_of_pty_output_thread = thread::spawn({
+            let reader = Arc::clone(&reader);
+            move || {
+                let mut reader = reader.lock().unwrap();
+                let mut buffer = Vec::new();
+                reader.read_to_end(&mut buffer).expect("finish reading pty");
+                String::from_utf8(buffer).unwrap()
+            }
+        });
+        child.wait()?;
+
+        let _ = read_remainder_of_pty_output_thread;
+        // Useful for debugging, but seems to deadlock on some tests:
+        // let remainder_of_pty_output = read_remainder_of_pty_output_thread.join().unwrap();
+        // assert!(
+        //     !remainder_of_pty_output.contains("panic"),
+        //     "Panic in PTY thread:\n{}",
+        //     console::strip_ansi_codes(&remainder_of_pty_output)
+        // );
+
+        Ok(())
+    }
+}
