@@ -65,9 +65,9 @@ use lib::util::ExitCode;
 use rayon::{ThreadPool, ThreadPoolBuilder};
 use tracing::{instrument, warn};
 
-use crate::commands::smartlog::smartlog;
-use crate::opts::{MoveOptions, ResolveRevsetOptions, Revset};
-use crate::revset::resolve_commits;
+use git_branchless_opts::{MoveOptions, ResolveRevsetOptions, Revset};
+use git_branchless_revset::resolve_commits;
+use git_branchless_smartlog::smartlog;
 use lib::core::config::get_restack_preserve_timestamps;
 use lib::core::dag::{commit_set_to_vec, union_all, CommitSet, Dag};
 use lib::core::effects::Effects;
@@ -90,7 +90,7 @@ fn restack_commits(
     event_cursor: EventCursor,
     git_run_info: &GitRunInfo,
     commits: Option<impl IntoIterator<Item = NonZeroOid>>,
-    build_options: &BuildRebasePlanOptions,
+    build_options: BuildRebasePlanOptions,
     execute_options: &ExecuteRebasePlanOptions,
     merge_conflict_remediation: MergeConflictRemediation,
 ) -> eyre::Result<ExitCode> {
@@ -103,8 +103,6 @@ fn restack_commits(
     // large and we'll be throwing away most of them.
     let commits = commit_set_to_vec(&commit_set)?;
 
-    let draft_commits = dag.query_draft_commits()?;
-
     struct RebaseInfo {
         dest_oid: NonZeroOid,
         abandoned_child_oids: Vec<NonZeroOid>,
@@ -112,13 +110,8 @@ fn restack_commits(
     let rebases: Vec<RebaseInfo> = {
         let mut result = Vec::new();
         for original_commit_oid in commits {
-            let abandoned_children = find_abandoned_children(
-                dag,
-                draft_commits,
-                event_replayer,
-                event_cursor,
-                original_commit_oid,
-            )?;
+            let abandoned_children =
+                find_abandoned_children(dag, event_replayer, event_cursor, original_commit_oid)?;
             if let Some((rewritten_oid, abandoned_child_oids)) = abandoned_children {
                 result.push(RebaseInfo {
                     dest_oid: rewritten_oid,
@@ -189,8 +182,8 @@ fn restack_commits(
             Ok(ExitCode(0))
         }
 
-        ExecuteRebasePlanResult::DeclinedToMerge { merge_conflict } => {
-            merge_conflict.describe(effects, &repo, merge_conflict_remediation)?;
+        ExecuteRebasePlanResult::DeclinedToMerge { failed_merge_info } => {
+            failed_merge_info.describe(effects, &repo, merge_conflict_remediation)?;
             Ok(ExitCode(1))
         }
 
@@ -334,6 +327,7 @@ pub fn restack(
         resolve_merge_conflicts,
         check_out_commit_options: CheckOutCommitOptions {
             additional_args: Default::default(),
+            reset: false,
             render_smartlog: false,
         },
     };
@@ -350,7 +344,7 @@ pub fn restack(
         event_cursor,
         git_run_info,
         commits,
-        &build_options,
+        build_options,
         &execute_options,
         merge_conflict_remediation,
     )?;
