@@ -1,6 +1,6 @@
 //! Implements the `git sync` command.
 
-use cursive::theme::BaseColor;
+use cursive_core::theme::BaseColor;
 use std::fmt::Write;
 use std::time::SystemTime;
 
@@ -12,17 +12,17 @@ use lib::core::repo_ext::RepoExt;
 use lib::util::ExitCode;
 use rayon::{ThreadPool, ThreadPoolBuilder};
 
-use crate::opts::{MoveOptions, ResolveRevsetOptions, Revset};
-use crate::revset::{check_revset_syntax, resolve_commits};
+use git_branchless_opts::{MoveOptions, ResolveRevsetOptions, Revset};
+use git_branchless_revset::{check_revset_syntax, resolve_commits};
 use lib::core::config::get_restack_preserve_timestamps;
 use lib::core::dag::{commit_set_to_vec, sorted_commit_set, union_all, CommitSet, Dag};
 use lib::core::effects::{Effects, OperationType};
 use lib::core::eventlog::{EventLogDb, EventReplayer};
-use lib::core::formatting::StyledStringBuilder;
+use lib::core::formatting::{Pluralize, StyledStringBuilder};
 use lib::core::rewrite::{
     execute_rebase_plan, BuildRebasePlanError, BuildRebasePlanOptions, ExecuteRebasePlanOptions,
-    ExecuteRebasePlanResult, RebasePlan, RebasePlanBuilder, RebasePlanPermissions, RepoPool,
-    RepoResource,
+    ExecuteRebasePlanResult, FailedMergeInfo, RebasePlan, RebasePlanBuilder, RebasePlanPermissions,
+    RepoPool, RepoResource,
 };
 use lib::core::task::ResourcePool;
 use lib::git::{CategorizedReferenceName, Commit, GitRunInfo, NonZeroOid, Repo};
@@ -89,6 +89,7 @@ pub fn sync(
         resolve_merge_conflicts,
         check_out_commit_options: CheckOutCommitOptions {
             additional_args: Default::default(),
+            reset: false,
             render_smartlog: false,
         },
     };
@@ -118,7 +119,7 @@ pub fn sync(
         git_run_info,
         &repo,
         &event_log_db,
-        &build_options,
+        build_options,
         &execute_options,
         &thread_pool,
         &repo_pool,
@@ -166,8 +167,7 @@ fn execute_main_branch_sync_plan(
         None => {
             writeln!(
                 effects.get_output_stream(),
-                "{} does not track an upstream branch, so not pulling.",
-                local_main_branch_description
+                "{local_main_branch_description} does not track an upstream branch, so not pulling."
             )?;
             return Ok(ExitCode(0));
         }
@@ -218,8 +218,7 @@ fn execute_main_branch_sync_plan(
     } else {
         writeln!(
             effects.get_output_stream(),
-            "Syncing {}",
-            local_main_branch_description
+            "Syncing {local_main_branch_description}"
         )?;
     }
 
@@ -231,7 +230,7 @@ fn execute_main_branch_sync_plan(
     };
     let permissions = match RebasePlanPermissions::verify_rewrite_set(
         &dag,
-        &build_options,
+        build_options,
         &local_main_branch_commits,
     )? {
         Ok(permissions) => permissions,
@@ -277,7 +276,7 @@ fn execute_sync_plans(
     git_run_info: &GitRunInfo,
     repo: &Repo,
     event_log_db: &EventLogDb,
-    build_options: &BuildRebasePlanOptions,
+    build_options: BuildRebasePlanOptions,
     execute_options: &ExecuteRebasePlanOptions,
     thread_pool: &ThreadPool,
     repo_pool: &ResourcePool<RepoResource>,
@@ -380,9 +379,9 @@ fn execute_plans(
     execute_options: &ExecuteRebasePlanOptions,
     root_commit_and_plans: Vec<(NonZeroOid, Option<RebasePlan>)>,
 ) -> Result<ExitCode, Report> {
-    let (success_commits, merge_conflict_commits, skipped_commits) = {
+    let (success_commits, failed_merge_commits, skipped_commits) = {
         let mut success_commits: Vec<Commit> = Vec::new();
-        let mut merge_conflict_commits: Vec<Commit> = Vec::new();
+        let mut failed_merge_commits: Vec<(Commit, FailedMergeInfo)> = Vec::new();
         let mut skipped_commits: Vec<Commit> = Vec::new();
 
         let (effects, progress) = effects.start_operation(OperationType::SyncCommits);
@@ -411,8 +410,8 @@ fn execute_plans(
                 ExecuteRebasePlanResult::Succeeded { rewritten_oids: _ } => {
                     success_commits.push(root_commit);
                 }
-                ExecuteRebasePlanResult::DeclinedToMerge { merge_conflict: _ } => {
-                    merge_conflict_commits.push(root_commit);
+                ExecuteRebasePlanResult::DeclinedToMerge { failed_merge_info } => {
+                    failed_merge_commits.push((root_commit, failed_merge_info));
                 }
                 ExecuteRebasePlanResult::Failed { exit_code } => {
                     return Ok(exit_code);
@@ -420,7 +419,7 @@ fn execute_plans(
             }
         }
 
-        (success_commits, merge_conflict_commits, skipped_commits)
+        (success_commits, failed_merge_commits, skipped_commits)
     };
 
     for success_commit in success_commits {
@@ -436,16 +435,39 @@ fn execute_plans(
         )?;
     }
 
-    for merge_conflict_commit in merge_conflict_commits {
-        writeln!(
-            effects.get_output_stream(),
-            "Merge conflict for {}",
-            effects.get_glyphs().render(
-                StyledStringBuilder::new()
-                    .append(merge_conflict_commit.friendly_describe(effects.get_glyphs())?)
-                    .build()
-            )?
-        )?;
+    for (failed_merge_commit, failed_merge_info) in failed_merge_commits {
+        match failed_merge_info {
+            FailedMergeInfo::Conflict {
+                commit_oid: _,
+                conflicting_paths,
+            } => {
+                writeln!(
+                    effects.get_output_stream(),
+                    "Merge conflict ({}) for {}",
+                    Pluralize {
+                        determiner: None,
+                        amount: conflicting_paths.len(),
+                        unit: ("file", "files")
+                    },
+                    effects.get_glyphs().render(
+                        StyledStringBuilder::new()
+                            .append(failed_merge_commit.friendly_describe(effects.get_glyphs())?)
+                            .build()
+                    )?
+                )?;
+            }
+            FailedMergeInfo::CannotRebaseMergeInMemory { commit_oid: _ } => {
+                writeln!(
+                    effects.get_output_stream(),
+                    "Can't rebase merge commit in-memory: {}",
+                    effects.get_glyphs().render(
+                        StyledStringBuilder::new()
+                            .append(failed_merge_commit.friendly_describe(effects.get_glyphs())?)
+                            .build()
+                    )?
+                )?;
+            }
+        }
     }
 
     for skipped_commit in skipped_commits {
