@@ -10,7 +10,12 @@
 //! changes should be staged for commit.
 
 #![warn(missing_docs)]
-#![warn(clippy::all, clippy::as_conversions, clippy::clone_on_ref_ptr)]
+#![warn(
+    clippy::all,
+    clippy::as_conversions,
+    clippy::clone_on_ref_ptr,
+    clippy::dbg_macro
+)]
 #![allow(clippy::too_many_arguments, clippy::blocks_in_if_conditions)]
 
 mod cursive_utils;
